//
//  EQLCIPCSocket.h
//  EQLibC
//
//  Created by Markus Hanauska on 08-04.
//  Copyright 2006 equinux AG. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#ifdef BUILD_EQ_LIBC_FRAMEWORK
#include "com/equinux/ipc/IPCSocket.h"
#endif

@interface EQLCIPCSocket : NSObject {
#ifdef BUILD_EQ_LIBC_FRAMEWORK
	com_equinux_ipc_Socket ipcSocket;
#else
	void * ipcSocket;
#endif

}

// On UNIX systems, name is a path, like /tmp/socketName
- (id) initWithName:(NSString *)name;

// Timeout is always in milliseconds (ms)
- (int) sendTimeout;
- (int) receiveTimeout;

// Returns the newly set timeout, which might be less than the requested one;
// any number < 0 means error.
- (int) setSendTimeout:(unsigned int)timeout;
- (int) setReceiveTimeout:(unsigned int)timeout;

// Connecting a socket and checking if it is already connected
- (NSError *) connect;
- (bool) isConnected;
// Connect may block very long, this method does not block longer than timeout
- (NSError *) connectWithTimeout:(unsigned int)timeout;

// A socket cannot be re-connected if it was connected and connection has been
// lost. Instead a new socket must be created each time.
- (bool) wasConnected;

// Socket disconnects automatically on dealloc
- (void) disconnect;

// Every socket has a unique ID, a 32 bit integer, but it's only unique within
// a certain amount of time; numbers might get re-used after a long time
- (u_int32_t) uniqueID;


/* Possible error codes for NSError:
 * 
 *  Unknown error:
 * 	CEN_(Socket_Exception_SocketError)
 *
 *  Operation would block, but socket is set to non-blocking:
 *  CEN_(Socket_Exception_WouldBlock)
 *
 *  The destination the socket tries to reach during connect is not reachable:
 *  CEN_(Socket_Exception_DestinationUnreachable)
 *
 *  The operation requested only works on a connected socket, this one is not:
 *  CEN_(Socket_Exception_NotConnected)
 *
 *  The last operation was interrupted by anonther proccess or thread:
 *  CEN_(Socket_Exception_SignalInterrupt)
 *
 *  A socket is already connected, but connection is requested:
 *  CEN_(Socket_Exception_IsAlreadyConnected)
 *
 *  The destination address of this socket is not valid:
 *  CEN_(Socket_Exception_InvalidAddress)
 *
 *  The system says you don't have sufficient privileges for this operation:
 *  CEN_(Socket_Exception_PermissionDenied)
 *
 * CEN_() is a macro that expands to "com_equinux_net_".
 * If no error happens, NSError return will be "nil".
 */

@end
