//
//  EQLCIPCSocketReader.h
//  EQLibC
//
//  Created by Markus Hanauska on 08-04.
//  Copyright 2006 equinux AG. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "EQLCIPCSocket.h"

#ifdef BUILD_EQ_LIBC_FRAMEWORK
#include "com/equinux/Thread.h"
#include "com/equinux/ipc/IPCSocketReader.h"
#endif

@interface EQLCIPCSocketReader : NSObject {
	
	id  delegate;
	SEL selector;
	volatile BOOL isAlive;
	volatile BOOL stayAlive;
	NSLock * aliveLock;
	EQLCIPCSocket * ipcSocket;
	
#ifdef BUILD_EQ_LIBC_FRAMEWORK
	com_equinux_Thread readerThread;
	com_equinux_ipc_SocketReader soReader;
#else
	void * readerThread;
	void * soReader;
#endif

}

// This message will call a selector of an object whenever a new message
// arrived; the parameter will be an object of type EQLCIPCIncomingMessage
// or of type NSError in case of an error.
- (id) initWithSocket:(EQLCIPCSocket *)soc delegate:(id)object
	andSelector:(SEL)sel expectedNumberOfMessages:(unsigned int)num;

// Age is in milliseconds (ms), incomplete packages are thrown away after
// that age. Default is several seconds for packages to complete.
- (id) initWithSocket:(EQLCIPCSocket *)soc maxPackageAge:(u_int32_t)age
	delegate:(id)object andSelector:(SEL)sel 
	expectedNumberOfMessages:(unsigned int)num;

/*
 * This class creates a new thread on init that runs in the background and
 * polls for incoming data traffic. If the selector is ever called with an
 * object of the kind NSError, this indicates that the socket is broken and
 * can't be used anymore (NSError will give details about what went wrong).
 * The thread will then automatically halt, no new packages can arrive.
 * In that case the the object should be dealloc'ed, the socket should be 
 * re-established and a new reader should be created.
 * The invocation of the delegate method always happens from the main thread,
 * not the spawned reader thread.
 *
 * Possible error codes for NSError:
 * 
 *  Unknown error:
 * 	CEN_(Socket_Exception_SocketError)
 *
 *  Operation would block, but socket is set to non-blocking:
 *  CEN_(Socket_Exception_WouldBlock)
 *
 *  The operation requested only works on a connected socket, this one is not:
 *  CEN_(Socket_Exception_NotConnected)
 *
 *  The last operation was interrupted by anonther proccess or thread:
 *  CEN_(Socket_Exception_SignalInterrupt)
 *
 *  The destination address of this socket is not valid:
 *  CEN_(Socket_Exception_InvalidAddress)
 *
 *  The system says you don't have sufficient privileges for this operation:
 *  CEN_(Socket_Exception_PermissionDenied)
 *
 * CEN_() is a macro that expands to "com_equinux_net_".
 * If no error happens, NSError return will be "nil".
 */


@end
